<?php

include_once('db.php');
include_once('general.php');

// Welcome to the checkout code.
// The first view of this is just like the cart, except values cannot be changed.
// To change a value, you simply go back to the cart.
// The preview does all the same clipping as the cart with verify_cart_values().
// It converts all the cart values from session vars to form vars.

// The actual checkout button verifies the quantities are available; if they
// aren't, the checkout fails, falling back to a preview where the quantities
// have been re-clipped, making the new page suitable for a retry.

// Connect to the db.
$db = dbconnect();

// Log a warning message if necessary.
$warning = NULL;
$order = NULL;

if( $_POST['action'] == 'checkout' )
{
	// This is the attempt to check out!
	// Get all the data.
	$count = get_int($_POST['count']) or 0;
	if( !$count ) {
		error("Cannot check out","The cart information was inconsistent.");
	}
	// Make sure this thing contains no duplicates.
	$items = array();
	// And make up a lovely where clause for the check
	$query = array();
	
	for($i=0;$i<$count;++$i) {
		$cat = get_int($_POST["cat$i"]);
		$item = get_int($_POST["item$i"]);
		$qty = get_int($_POST["qty$i"]);
		$dup = 0;
		if( isset($items["$cat-$item"]) ) {
			// trouble
			$dup = 1;
		} else {
			// don't let it repeat
			$items["$cat-$item"] = $qty;
		}
		if( !(isset($cat)&&isset($item)&&isset($qty)) or
			$cat < 0 or $item < 0 or $qty <= 0 or $dup )
		{
			error("Cannot check out","The cart information was inconsistent.");
		}

		// Lookin' good.
		$query[] = sprintf("(p.category_id='%s' and p.product_id='%s' and p.quantity>='%s')",
			dbquote($cat,$db),dbquote($item,$db),dbquote($qty,$db) );
	}

	// Construct our quantity checker
	$query = "select p.category_id,p.product_id,p.quantity,u.name,p.name from product p, unit u " .
		"where u.unit_id = p.unit_id and (" .
		implode(" or ", $query) . ") order by p.name";

	$r = dbquery($query,$db);

	// If the row count is wrong, there's not enough of everything.
	if( $count != mysql_num_rows($r) ) {
		$warning = "The stock for one or more of your requested items is insufficient to fill " .
			"your order.";
	}
	else {
		// Grand finale!
		$query = array();
		$order = array();
		while( list($cat,$item,$quantity,$unit,$name) = mysql_fetch_array($r) ) {
			$order[] = array($cat,$item,$name,$items["$cat-$item"],$unit);
			// decrement by the number we wanted.
			$new_quantity = $quantity - $items["$cat-$item"];
			// Write a query for this.
			// The extra condition attempts safety on a change-by-change basis.
			$query[] = sprintf("update product set quantity='%s' where " .
				" category_id='%s' and product_id='%s' and quantity='%s'",
				dbquote($new_quantity,$db),dbquote($cat,$db),dbquote($item,$db),
				dbquote($quantity,$db) );
		}

		// Execute the queries...execution-style
		foreach($query as $q) {
			$r = dbquery($q,$db);
		}

		// That should be it!
		empty_cart();
	}	
}

// We'll use a first query to clean up nonexistent items
// and items whose quantities are over cap.
verify_cart_values($db);

// Now, the cart content should be mostly cleaned up.
// Seek the item info to display.
$cart_empty = 0;
if( isset($_SESSION['cart']) and count($_SESSION['cart']) > 0 ) {
	// Once again, the items are the keys of this array.
	$query = array();
	{
		foreach(array_keys($_SESSION['cart']) as $key) {
			// Split the key
			list($cat,$item) = explode('-',$key,2);
			$cat = get_int($cat) or 0;
			$item = get_int($item) or 0;
			// add to the where clause
			$query[] = sprintf("(p.category_id='%s' and p.product_id='%s')",
				dbquote($cat,$db),dbquote($item,$db) );
		}
	}
	// Assemble the query
	$query = "select p.category_id,p.product_id,p.name,p.quantity,u.name
		from product p, unit u
		where u.unit_id = p.unit_id
		and (" . implode(" or ",$query) . ")
		order by p.name";

	$r = dbquery($query,$db);
} else {
	$cart_empty = 1;
}

$page_title = "Checkout";


function checkout_succeeded() {
	global $order;

	?>
	<h1>Congratulations!</h1>
	<p>Your pseudo-purchase has succeeded.  Thank you for pseudo-shopping!</p>

	<h2>Your order</h2>
	<table>
	<tr>
	<th colspan="2">Product</th>
	<th>Quantity</th>
	</tr>
	<?php

	// What was ordered?
	foreach($order as $order_item) {
		list($cat,$item,$name,$quantity,$unit) = $order_item;
		$n = htmlspecialchars($name);
		print "<tr>";
		print "<td><img src='" . image_loc($cat,$item,1) . "' alt='$n' /></td>";
		print "<td>$n</td>";
		print "<td>$quantity $unit</td>";
		print "</tr>";
	}

	?>
	</table>

	<?php
}


class page {

function body() {
	global $r, $cart_empty, $warning, $order;

	if( !isset($warning) and isset($order) ) {
		checkout_succeeded();
		return;
	}

	?>
	<h1>Checkout<?php if(isset($warning)) print " Error" ?></h1>

	<?php if(isset($warning)) print "<p>" . htmlspecialchars($warning) . "</p>"; ?>


	<div>
<?php
	if( $cart_empty || mysql_num_rows($r) == 0 ) {
		?><p>There are no items in the cart.</p><?php
	}
	else {
		?>
		<p>Verify the contents of your cart before continuing.  If you want to change
		quantities, please <?php print "<a href='" . base_loc("cart") . "'>return to the cart</a>"; ?>.</p>

		<div>
		<?php print "<form action='" . base_loc("checkout") . "' ".
			"method='post' >"; ?>
		<input type="hidden" name="action" value="checkout" />
		<?php print "<input type='hidden' name='count' value='" .
			mysql_num_rows($r) . "' />" ?>
		<table>
		<tr>
			<th colspan="2">Product</th>
			<th>Stock</th>
			<th>In cart</th>
		</tr>
		<?php

		$iter = 0;

		while(list($cid,$pid,$pname,$qty,$unit)=mysql_fetch_array($r)) {
			// Print each item as a table row
			$loc = item_loc($cid,$pid);
			$img = image_loc($cid,$pid,1);
			$pname = htmlspecialchars($pname);
			$link = "<a href='$loc'>";
			?>
			<tr>
			<td><?php print "$link<img src='$img' alt='$pname' /></a>" ?></td>
			<td><?php print "$link$pname</a>" ?></td>
			<td><?php
				// None left?
				if($qty == 0) {
					print "Sold Out";
				}
				else {
					// We have some...
					$qtywords = "$qty $unit";
					// But not many...
					if($qty < 10) {
						print "Hurry! Only $qtywords left in stock!";
					} else {
						// Or plenty.
						print $qtywords;
					}
				}
			?></td>
			<td><?php
				// Form information in this cell.
				$h = "<input type='hidden' name=";
				print "$h'cat$iter' value='$cid' />";
				print "$h'item$iter' value='$pid' />";
				$incart = min(in_cart($cid,$pid),$qty);
				print "<input type='hidden' name='qty$iter' size='3' value='$incart' />";
				print "$incart $unit";
			?></td>
			</tr><?php

			// Increment counter.
			++$iter;
		}

		?>
		<tr><td colspan="4">
		<input type="submit" value="Check out" />
		</td></tr>
		</table></form></div>

		<?php

	}


?>
	</div>


	<?php
}

}

$page = new page();
include_once('template.php');

?>
