<?php

// Tell our program where it is
$PROGRAM_BASE = '/static/school/store/';

// Enable sessions
session_start();

// Avoid register_globals trouble
unset($page_title);

function get_arguments() {
	// Returns the part of the URL after the script name,
	// in pieces.
	$script = $_SERVER['SCRIPT_NAME'];
	$list = $_SERVER['SCRIPT_URL'];

	$script = preg_split('/\\/+/', $script, 16, PREG_SPLIT_NO_EMPTY);
	$list = preg_split('/\\/+/', $list, 16, PREG_SPLIT_NO_EMPTY);

	while(isset($script[0]) and isset($list[0]) and
		$script[0] == $list[0])
	{
		array_shift($script);
		array_shift($list);
	}

	return $list;
}

function get_int($n) {
	// Returns $n as an integer, but only if
	// $n is an integer to begin with
	if(!is_numeric($n))
		return NULL;
	else if(intval($n) != $n)
		return NULL;
	else
		return intval($n);
}

function error($title,$description) {
	// Creates an error page
	global $ERROR_TITLE,$ERROR_DESCRIPTION;
	$ERROR_TITLE = $title;
	$ERROR_DESCRIPTION = htmlspecialchars($description);
	include_once('error.php');
}

function base_loc($plus) {
	global $PROGRAM_BASE;
	return $PROGRAM_BASE . $plus;
}

function image_loc($cat,$num,$thumb) {
	// returns the location of the image
	$thumb = $thumb ? "-small" : "";
	return base_loc("images/$cat-$num$thumb");
}

function item_loc($cat,$num) {
	// returns the location of the item
	return base_loc("item/$cat/$num");
}

function category_loc($cat) {
	// returns the location of the category
	return base_loc("category/$cat");
}

function add_loc($cat,$num) {
	// return location to add one to inventory
	return base_loc("cart/add/$cat/$num");
}

function remove_loc($cat,$num) {
	// return location to remove all from inventory
	return base_loc("cart/remove/$cat/$num");
}

function in_cart($cat,$num) {
	// returns the number of the item in the cart
	// (according to the cookie)
	$n = get_int( $_SESSION['cart']["$cat-$num"] );
	if(!isset($n) or $n < 0)
		$n = 0;
	return $n;
}

function change_in_cart($cat,$num,$qty) {
	// changes the quantity in cart
	$qty = get_int($qty);
	if(!isset($qty)) $qty = 0;
	$qty = max($qty,0);
	if($qty == 0)
		remove_from_cart($cat,$num);
	else
		$_SESSION['cart']["$cat-$num"] = $qty;
}

function add_to_cart($cat,$num) {
	// adds one to the number in cart
	$n = in_cart($cat,$num);
	$_SESSION['cart']["$cat-$num"] = $n + 1;
}

function remove_from_cart($cat,$num) {
	// removes all from cart
	unset($_SESSION['cart']["$cat-$num"]);
}

function empty_cart() {
	// guess
	unset($_SESSION['cart']);
}

function verify_cart_values($db) {
	// This function is used from cart and checkout
	// to verify that all items in the cart exist
	// and caps the cart quantities to those available.
	if( isset($_SESSION['cart']) and count($_SESSION['cart']) > 0 ) {
		// The items are the keys of this array.
		$items = array();
		$query = array();
		{
			$itemkeys = array_keys($_SESSION['cart']);
			foreach($itemkeys as $key) {
				// Split the key back into numbers
				list($cat,$item) = explode('-',$key,2);
				$cat = get_int($cat) or 0;
				$item = get_int($item) or 0;
				$items[$key] = 1;
				// add to the where clause
				$query[] = sprintf("(p.category_id='%s' and p.product_id='%s')",
					dbquote($cat,$db),dbquote($item,$db) );
			}
		}
		// All right, now put together the real query
		$query = "select p.category_id,p.product_id,p.quantity ".
			"from product p where " .
			implode(" or ", $query);

		$r = dbquery($query,$db);

		// We'll cap the things that need capping.
		while(list($cat,$item,$qty)=mysql_fetch_array($r)) {
			if( in_cart($cat,$item) > $qty )
				change_in_cart($cat,$item,$qty);

			// Also, record that we found this record in
			// common with what is in the session.
			unset($items["$cat-$item"]);
		}

		// If we didn't find the item at all,
		// remove the row from the cart.
		foreach(array_keys($items) as $key) {
			unset($_SESSION['cart'][$key]);
		}
	}
}
?>
